/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * MoleculeByNameAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/MoleculeNameDescriptor.hpp"
#include "Descriptors/MoleculeSelectionDescriptor.hpp"
#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "MoleculeByNameAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "MoleculeByNameAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr SelectionMoleculeByNameAction::performCall() {
  LOG(1, "Selecting all molecules called " << params.molname.get());
  std::vector<molecule *> selectedMolecules =
      World::getInstance().getAllMolecules(MoleculesBySelection() && MoleculeByName(params.molname.get()));
  World::getInstance().selectAllMolecules(MoleculeByName(params.molname.get()));
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");

  return Action::state_ptr(new SelectionMoleculeByNameState(selectedMolecules,params));
}

Action::state_ptr SelectionMoleculeByNameAction::performUndo(Action::state_ptr _state) {
  SelectionMoleculeByNameState *state = assert_cast<SelectionMoleculeByNameState*>(_state.get());

  World::getInstance().unselectAllMolecules(MoleculeByName(state->params.molname.get()));
  BOOST_FOREACH( molecule *mol, state->selectedMolecules)
    World::getInstance().selectMolecule(mol);

  return Action::state_ptr(_state);
}

Action::state_ptr SelectionMoleculeByNameAction::performRedo(Action::state_ptr _state){
  SelectionMoleculeByNameState *state = assert_cast<SelectionMoleculeByNameState*>(_state.get());

  World::getInstance().selectAllMolecules(MoleculeByName(state->params.molname.get()));

  return Action::state_ptr(_state);
}

bool SelectionMoleculeByNameAction::canUndo() {
  return true;
}

bool SelectionMoleculeByNameAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
